# Sebastian Raschka 2014-2024
# mlxtend Machine Learning Library Extensions
#
# Implementation of the mulitnomial logistic regression algorithm for
# classification.

# Author: Sebastian Raschka <sebastianraschka.com>
#
# License: BSD 3 clause

from time import time

import numpy as np

from .._base import _BaseModel, _Classifier, _IterativeModel, _MultiClass


class SoftmaxRegression(_BaseModel, _IterativeModel, _Classifier, _MultiClass):

    """Softmax regression classifier.

    Parameters
    ------------
    eta : float (default: 0.01)
        Learning rate (between 0.0 and 1.0)

    epochs : int (default: 50)
        Passes over the training dataset.
        Prior to each epoch, the dataset is shuffled
        if `minibatches > 1` to prevent cycles in stochastic gradient descent.

    l2 : float
        Regularization parameter for L2 regularization.
        No regularization if l2=0.0.

    minibatches : int (default: 1)
        The number of minibatches for gradient-based optimization.
        If 1: Gradient Descent learning
        If len(y): Stochastic Gradient Descent (SGD) online learning
        If 1 < minibatches < len(y): SGD Minibatch learning

    n_classes : int (default: None)
        A positive integer to declare the number of class labels
        if not all class labels are present in a partial training set.
        Gets the number of class labels automatically if None.

    random_seed : int (default: None)
        Set random state for shuffling and initializing the weights.

    print_progress : int (default: 0)
        Prints progress in fitting to stderr.
        0: No output
        1: Epochs elapsed and cost
        2: 1 plus time elapsed
        3: 2 plus estimated time until completion

    Attributes
    -----------
    w_ : 2d-array, shape={n_features, 1}
      Model weights after fitting.

    b_ : 1d-array, shape={1,}
      Bias unit after fitting.

    cost_ : list
        List of floats, the average cross_entropy for each epoch.

    Examples
    -----------
    For usage examples, please see
    https://rasbt.github.io/mlxtend/user_guide/classifier/SoftmaxRegression/

    """

    def __init__(
        self,
        eta=0.01,
        epochs=50,
        l2=0.0,
        minibatches=1,
        n_classes=None,
        random_seed=None,
        print_progress=0,
    ):
        _BaseModel.__init__(self)
        _IterativeModel.__init__(self)
        _Classifier.__init__(self)
        _MultiClass.__init__(self)

        self.eta = eta
        self.epochs = epochs
        self.l2 = l2
        self.minibatches = minibatches
        self.n_classes = n_classes
        self.random_seed = random_seed
        self.print_progress = print_progress
        self._is_fitted = False

    def _net_input(self, X):
        return X.dot(self.w_) + self.b_

    def _softmax_activation(self, z):
        e_x = np.exp(z - z.max(axis=1, keepdims=True))
        out = e_x / e_x.sum(axis=1, keepdims=True)
        return out
        # return (np.exp(z.T) / np.sum(np.exp(z), axis=1)).T

    def _cross_entropy(self, output, y_target):
        return -np.sum(np.log(output) * (y_target), axis=1)

    def _cost(self, cross_entropy):
        L2_term = self.l2 * np.sum(self.w_**2)
        cross_entropy = cross_entropy + L2_term
        return 0.5 * np.mean(cross_entropy)

    def _to_classlabels(self, z):
        return z.argmax(axis=1)

    def _forward(self, X):
        z = self._net_input(X)
        a = self._softmax_activation(z)
        return a

    def _backward(self, X, y_true, y_probas):
        grad_loss_wrt_out = y_true - y_probas
        # gradient -> n_features x n_classes
        grad_loss_wrt_w = -np.dot(X.T, grad_loss_wrt_out)
        grad_loss_wrt_b = -np.sum(grad_loss_wrt_out, axis=0)
        return grad_loss_wrt_w, grad_loss_wrt_b

    def _fit(self, X, y, init_params=True):
        self._check_target_array(y)
        if init_params:
            if self.n_classes is None:
                self.n_classes = np.max(y) + 1
            self._n_features = X.shape[1]

            self.b_, self.w_ = self._init_params(
                weights_shape=(self._n_features, self.n_classes),
                bias_shape=(self.n_classes,),
                random_seed=self.random_seed,
            )
            self.cost_ = []

        y_enc = self._one_hot(y=y, n_labels=self.n_classes, dtype=np.float_)

        self.init_time_ = time()
        rgen = np.random.RandomState(self.random_seed)
        for i in range(self.epochs):
            for idx in self._yield_minibatches_idx(
                rgen=rgen, n_batches=self.minibatches, data_ary=y, shuffle=True
            ):
                # net_input, softmax and diff -> n_samples x n_classes:
                y_probas = self._forward(X[idx])

                # w_ -> n_feat x n_classes
                # b_  -> n_classes
                grad_loss_wrt_w, grad_loss_wrt_b = self._backward(
                    X[idx], y_true=y_enc[idx], y_probas=y_probas
                )

                # update in opp. direction of the cost gradient
                l2_reg = self.l2 * self.w_
                self.w_ += self.eta * (-grad_loss_wrt_w - l2_reg)
                self.b_ += self.eta * -grad_loss_wrt_b

            # compute cost of the whole epoch
            y_probas = self._forward(X)
            cross_ent = self._cross_entropy(output=y_probas, y_target=y_enc)
            cost = self._cost(cross_ent)
            self.cost_.append(cost)

            if self.print_progress:
                self._print_progress(iteration=i + 1, n_iter=self.epochs, cost=cost)

        return self

    def predict_proba(self, X):
        """Predict class probabilities of X from the net input.

        Parameters
        ----------
        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples and
            n_features is the number of features.

        Returns
        ----------
        Class probabilties : array-like, shape= [n_samples, n_classes]

        """
        return self._forward(X)

    def _predict(self, X):
        probas = self.predict_proba(X)
        return self._to_classlabels(probas)



# Sebastian Raschka 2014-2024
# mlxtend Machine Learning Library Extensions
# Author: Sebastian Raschka <sebastianraschka.com>
#
# License: BSD 3 clause

import sys

from packaging.version import Version
from sklearn import __version__ as sklearn_version
from sklearn.linear_model import Lasso, LogisticRegression, Ridge
from sklearn.model_selection import train_test_split
from sklearn.tree import DecisionTreeClassifier

from mlxtend.data import boston_housing_data, iris_data
from mlxtend.evaluate import paired_ttest_resampled
from mlxtend.utils import assert_raises


def test_train_size():
    X, y = iris_data()
    clf1 = LogisticRegression(solver="liblinear", multi_class="ovr")
    clf2 = DecisionTreeClassifier()

    expected_err_msg = (
        "train_size must be of type int or float. " "Got <class 'NoneType'>."
    )

    if sys.version_info < (3, 0):
        expected_err_msg = expected_err_msg.replace("<class", "<type")

    assert_raises(
        ValueError,
        expected_err_msg,
        paired_ttest_resampled,
        clf1,
        clf2,
        X,
        y,
        test_size=None,
    )


def test_classifier_defaults():
    X, y = iris_data()
    clf1 = LogisticRegression(multi_class="ovr", solver="liblinear", random_state=1)
    clf2 = DecisionTreeClassifier(random_state=1)

    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=0.25, random_state=123
    )

    score1 = clf1.fit(X_train, y_train).score(X_test, y_test)
    score2 = clf2.fit(X_train, y_train).score(X_test, y_test)

    assert round(score1, 2) == 0.97
    assert round(score2, 2) == 0.95

    t, p = paired_ttest_resampled(
        estimator1=clf1, estimator2=clf2, X=X, y=y, random_seed=1
    )

    if Version(sklearn_version) < Version("0.20"):
        assert round(t, 3) == -1.809, t
        assert round(p, 3) == 0.081, p
    else:
        assert round(t, 3) == -1.702, t
        assert round(p, 3) == 0.10, p

    # change maxdepth of decision tree classifier

    clf2 = DecisionTreeClassifier(max_depth=1, random_state=1)

    score3 = clf2.fit(X_train, y_train).score(X_test, y_test)

    assert round(score3, 2) == 0.63

    t, p = paired_ttest_resampled(
        estimator1=clf1, estimator2=clf2, X=X, y=y, random_seed=1
    )

    assert round(t, 3) == 39.214, t
    assert round(p, 3) == 0.000, p


def test_scoring():
    X, y = iris_data()
    clf1 = LogisticRegression(multi_class="ovr", solver="liblinear", random_state=1)
    clf2 = DecisionTreeClassifier(random_state=1)

    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=0.25, random_state=123
    )

    score1 = clf1.fit(X_train, y_train).score(X_test, y_test)
    score2 = clf2.fit(X_train, y_train).score(X_test, y_test)

    assert round(score1, 2) == 0.97
    assert round(score2, 2) == 0.95

    t, p = paired_ttest_resampled(
        estimator1=clf1, estimator2=clf2, X=X, y=y, scoring="accuracy", random_seed=1
    )

    if Version(sklearn_version) < Version("0.20"):
        assert round(t, 3) == -1.809, t
        assert round(p, 3) == 0.081, p
    else:
        assert round(t, 3) == -1.702, t
        assert round(p, 3) == 0.1, p

    t, p = paired_ttest_resampled(
        estimator1=clf1, estimator2=clf2, X=X, y=y, scoring="f1_macro", random_seed=1
    )

    if Version(sklearn_version) < Version("0.20"):
        assert round(t, 3) == -1.690, t
        assert round(p, 3) == 0.102, p
    else:
        assert round(t, 3) == -1.561, t
        assert round(p, 3) == 0.129, p


def test_regressor():
    X, y = boston_housing_data()
    reg1 = Lasso(random_state=1)
    reg2 = Ridge(random_state=1)

    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=0.25, random_state=123
    )

    score1 = reg1.fit(X_train, y_train).score(X_test, y_test)
    score2 = reg2.fit(X_train, y_train).score(X_test, y_test)

    assert round(score1, 2) == 0.66, score1
    assert round(score2, 2) == 0.68, score2

    t, p = paired_ttest_resampled(
        estimator1=reg1, estimator2=reg2, X=X, y=y, random_seed=1
    )

    assert round(t, 3) == -7.697, t
    assert round(p, 3) == 0.000, p



# Sebastian Raschka 2014-2024
# mlxtend Machine Learning Library Extensions
# Author: Sebastian Raschka <sebastianraschka.com>
#
# License: BSD 3 clause

import numpy as np
from sklearn.linear_model import LogisticRegression

from mlxtend.classifier import OneRClassifier
from mlxtend.data import iris_data
from mlxtend.evaluate import create_counterfactual
from mlxtend.utils import assert_raises


def test__medium_lambda():
    X, y = iris_data()
    clf = LogisticRegression()
    clf.fit(X, y)

    x_ref = X[15]

    res = create_counterfactual(
        x_reference=x_ref,
        y_desired=2,
        model=clf,
        X_dataset=X,
        y_desired_proba=1.0,
        lammbda=1,
        random_seed=123,
    )

    assert np.argmax(clf.predict_proba(x_ref.reshape(1, -1))) == 0
    assert np.argmax(clf.predict_proba(res.reshape(1, -1))) == 2
    assert (
        round((clf.predict_proba(0.65 >= res.reshape(1, -1))).flatten()[-1], 2) <= 0.69
    )


def test__small_lambda():
    X, y = iris_data()
    clf = LogisticRegression()
    clf.fit(X, y)

    x_ref = X[15]

    res = create_counterfactual(
        x_reference=x_ref,
        y_desired=2,
        model=clf,
        X_dataset=X,
        y_desired_proba=1.0,
        lammbda=0.0001,
        random_seed=123,
    )

    assert np.argmax(clf.predict_proba(x_ref.reshape(1, -1))) == 0
    assert np.argmax(clf.predict_proba(res.reshape(1, -1))) == 0
    assert round((clf.predict_proba(res.reshape(1, -1))).flatten()[-1], 2) == 0.0


def test__large_lambda():
    X, y = iris_data()
    clf = LogisticRegression()
    clf.fit(X, y)

    x_ref = X[15]

    res = create_counterfactual(
        x_reference=x_ref,
        y_desired=2,
        model=clf,
        X_dataset=X,
        y_desired_proba=1.0,
        lammbda=100,
        random_seed=123,
    )

    assert np.argmax(clf.predict_proba(x_ref.reshape(1, -1))) == 0
    assert np.argmax(clf.predict_proba(res.reshape(1, -1))) == 2
    assert round((clf.predict_proba(res.reshape(1, -1))).flatten()[-1], 2) >= 0.96


def test__clf_with_no_proba_fail():
    X, y = iris_data()
    clf = OneRClassifier()
    clf.fit(X, y)

    x_ref = X[15]

    s = (
        "Your `model` does not support "
        "`predict_proba`. Set `y_desired_proba` "
        " to `None` to use `predict`instead."
    )

    assert_raises(
        AttributeError, s, create_counterfactual, x_ref, 2, clf, X, 1.0, 100, 123
    )


def test__clf_with_no_proba_pass():
    X, y = iris_data()
    clf = OneRClassifier()
    clf.fit(X, y)

    x_ref = X[15]

    res = create_counterfactual(
        x_reference=x_ref,
        y_desired=2,
        model=clf,
        X_dataset=X,
        y_desired_proba=None,
        lammbda=100,
        random_seed=123,
    )

    assert clf.predict(x_ref.reshape(1, -1)) == 0
    assert clf.predict(res.reshape(1, -1)) == 2



from mlxtend.text import tokenizer_emoticons, tokenizer_words_and_emoticons


def test_tokenizer_words_and_emoticons_1():
    assert tokenizer_words_and_emoticons("</a>This :) is :( a test :-)!") == [
        "this",
        "is",
        "a",
        "test",
        ":)",
        ":(",
        ":-)",
    ]


def test_tokenizer_words_and_emoticons_2():
    assert tokenizer_emoticons("</a>This :) is :( a test :-)!") == [":)", ":(", ":-)"]



# Sebastian Raschka 2014-2024
# mlxtend Machine Learning Library Extensions
# Author: Sebastian Raschka <sebastianraschka.com>
#
# License: BSD 3 clause


import numpy as np

from mlxtend.preprocessing import shuffle_arrays_unison


def test_shuffle_arrays_unison():
    X1 = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
    y1 = np.array([1, 2, 3])

    X2, y2 = shuffle_arrays_unison(arrays=[X1, y1], random_seed=3)

    assert X2.all() == np.array([[4, 5, 6], [1, 2, 3], [7, 8, 9]]).all()
    assert y2.all() == np.array([2, 1, 3]).all()



import os
import re

import numpy as np
import pytest
from numpy.testing import assert_array_equal

from mlxtend.data import iris_data

this_dir, this_filename = os.path.split(__file__)
this_dir = re.sub("^stset", "", this_dir[::-1])[::-1]
DATA_PATH = os.path.join(this_dir, "data", "iris.csv.gz")


def test_iris_data_uci():
    tmp = np.genfromtxt(fname=DATA_PATH, delimiter=",")
    original_uci_data_x, original_uci_data_y = tmp[:, :-1], tmp[:, -1]
    original_uci_data_y = original_uci_data_y.astype(int)
    iris_x, iris_y = iris_data()
    assert_array_equal(original_uci_data_x, iris_x)
    assert_array_equal(original_uci_data_y, iris_y)


def test_iris_data_r():
    tmp = np.genfromtxt(fname=DATA_PATH, delimiter=",")
    original_r_data_x, original_r_data_y = tmp[:, :-1], tmp[:, -1]
    original_r_data_y = original_r_data_y.astype(int)
    original_r_data_x[34] = [4.9, 3.1, 1.5, 0.2]
    original_r_data_x[37] = [4.9, 3.6, 1.4, 0.1]
    iris_x, iris_y = iris_data(version="corrected")
    assert_array_equal(original_r_data_x, iris_x)


def test_iris_invalid_choice():
    with pytest.raises(ValueError) as excinfo:
        iris_data(version="bla")
        assert excinfo.value.message == "version must be 'uci' or 'corrected'."



import pandas as pd

from mlxtend.data import iris_data
from mlxtend.plotting import scatter_hist

X, y = iris_data()
df = pd.DataFrame(X)
df.columns = [
    "sepal length [cm]",
    "sepal width [cm]",
    "petal length [cm]",
    "petal width [cm]",
]


def test_pass_data_as_dataframe():
    scatter_hist(df["sepal length [cm]"], df["sepal width [cm]"])


def test_pass_data_as_numpy_array():
    scatter_hist(X[:, 0], X[:, 1])



# mlxtend Machine Learning Library Extensions
#
# Time series cross validation with grouping.
# Author: Dmitry Labazkin <labdmitriy@gmail.com>
#
# License: BSD 3 clause

import numpy as np
import pytest
from sklearn.dummy import DummyClassifier
from sklearn.model_selection import cross_val_score

from mlxtend.evaluate import GroupTimeSeriesSplit


@pytest.fixture
def X():
    return np.array(
        [[0], [7], [6], [4], [4], [8], [0], [6], [2], [0], [5], [9], [7], [7], [7], [7]]
    )


@pytest.fixture
def y():
    return np.array([1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0])


@pytest.fixture
def group_numbers():
    return np.array([0, 1, 1, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 5, 5])


@pytest.fixture
def not_sorted_group_numbers():
    return np.array([5, 5, 5, 5, 1, 1, 1, 1, 3, 3, 2, 2, 2, 4, 4, 0])


@pytest.fixture
def not_consecutive_group_numbers():
    return np.array([0, 1, 1, 1, 1, 2, 2, 2, 3, 3, 4, 4, 2, 2, 2, 2])


@pytest.fixture
def group_names():
    return np.array(
        [
            "2021-01",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-03",
            "2021-03",
            "2021-03",
            "2021-04",
            "2021-04",
            "2021-05",
            "2021-05",
            "2021-06",
            "2021-06",
            "2021-06",
            "2021-06",
        ]
    )


@pytest.fixture
def not_sorted_group_names():
    return np.array(
        [
            "2021-06",
            "2021-06",
            "2021-06",
            "2021-06",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-04",
            "2021-04",
            "2021-03",
            "2021-03",
            "2021-03",
            "2021-05",
            "2021-05",
            "2021-01",
        ]
    )


@pytest.fixture
def not_consecutive_group_names():
    return np.array(
        [
            "2021-01",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-02",
            "2021-03",
            "2021-03",
            "2021-03",
            "2021-04",
            "2021-04",
            "2021-05",
            "2021-05",
            "2021-03",
            "2021-03",
            "2021-03",
            "2021-03",
        ]
    )


def check_splits(X, y, groups, cv_args, expected_results):
    cv = GroupTimeSeriesSplit(**cv_args)
    results = list(cv.split(X, y, groups))

    assert len(results) == len(expected_results)

    for split, expected_split in zip(results, expected_results):
        assert np.array_equal(split[0], expected_split[0])
        assert np.array_equal(split[1], expected_split[1])

    return cv


def test_get_n_splits(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4, 5, 6, 7]), np.array([8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7, 8, 9]), np.array([10, 11])),
        (np.array([5, 6, 7, 8, 9, 10, 11]), np.array([12, 13, 14, 15])),
    ]
    cv = check_splits(X, y, group_numbers, cv_args, expected_results)

    assert cv.get_n_splits() == len(expected_results)


def test_train_size(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4, 5, 6, 7]), np.array([8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7, 8, 9]), np.array([10, 11])),
        (np.array([5, 6, 7, 8, 9, 10, 11]), np.array([12, 13, 14, 15])),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_train_size_group_names(X, y, group_names):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4, 5, 6, 7]), np.array([8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7, 8, 9]), np.array([10, 11])),
        (np.array([5, 6, 7, 8, 9, 10, 11]), np.array([12, 13, 14, 15])),
    ]
    check_splits(X, y, group_names, cv_args, expected_results)


def test_n_splits(X, y, group_numbers):
    cv_args = {"test_size": 2, "n_splits": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4]), np.array([5, 6, 7, 8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7]), np.array([8, 9, 10, 11])),
        (np.array([5, 6, 7, 8, 9]), np.array([10, 11, 12, 13, 14, 15])),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_n_splits_gap_size(X, y, group_numbers):
    cv_args = {"test_size": 1, "n_splits": 3, "gap_size": 1}
    expected_results = [
        (np.array([0, 1, 2, 3, 4]), np.array([8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7]), np.array([10, 11])),
        (np.array([5, 6, 7, 8, 9]), np.array([12, 13, 14, 15])),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_n_splits_shift_size(X, y, group_numbers):
    cv_args = {"test_size": 1, "n_splits": 3, "gap_size": 1}
    expected_results = [
        (np.array([0, 1, 2, 3, 4]), np.array([8, 9])),
        (np.array([1, 2, 3, 4, 5, 6, 7]), np.array([10, 11])),
        (np.array([5, 6, 7, 8, 9]), np.array([12, 13, 14, 15])),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_n_splits_expanding_window(X, y, group_numbers):
    cv_args = {"test_size": 3, "n_splits": 3, "window_type": "expanding"}
    expected_results = [
        (np.array([0]), np.array([1, 2, 3, 4, 5, 6, 7, 8, 9])),
        (np.array([0, 1, 2, 3, 4]), np.array([5, 6, 7, 8, 9, 10, 11])),
        (
            np.array([0, 1, 2, 3, 4, 5, 6, 7]),
            np.array([8, 9, 10, 11, 12, 13, 14, 15]),
        ),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_full_usage_of_data(X, y, group_numbers):
    cv_args = {"test_size": 3, "train_size": 2, "n_splits": 2}
    expected_results = [
        (np.array([0, 1, 2, 3, 4]), np.array([5, 6, 7, 8, 9, 10, 11])),
        (
            np.array([1, 2, 3, 4, 5, 6, 7]),
            np.array([8, 9, 10, 11, 12, 13, 14, 15]),
        ),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_partial_usage_of_data(X, y, group_numbers):
    cv_args = {"test_size": 2, "train_size": 2, "n_splits": 2}
    expected_results = [
        (np.array([1, 2, 3, 4, 5, 6, 7]), np.array([8, 9, 10, 11])),
        (np.array([5, 6, 7, 8, 9]), np.array([10, 11, 12, 13, 14, 15])),
    ]
    check_splits(X, y, group_numbers, cv_args, expected_results)


def test_not_sorted_group_numbers(X, y, not_sorted_group_numbers):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9]), np.array([10, 11, 12])),
        (np.array([4, 5, 6, 7, 8, 9, 10, 11, 12]), np.array([13, 14])),
        (np.array([8, 9, 10, 11, 12, 13, 14]), np.array([15])),
    ]

    check_splits(X, y, not_sorted_group_numbers, cv_args, expected_results)


def test_not_sorted_group_names(X, y, not_sorted_group_names):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = [
        (np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9]), np.array([10, 11, 12])),
        (np.array([4, 5, 6, 7, 8, 9, 10, 11, 12]), np.array([13, 14])),
        (np.array([8, 9, 10, 11, 12, 13, 14]), np.array([15])),
    ]

    check_splits(X, y, not_sorted_group_names, cv_args, expected_results)


def test_not_specified_train_size_n_splits(X, y, group_numbers):
    cv_args = {"test_size": 1}
    expected_results = None
    error_message = "Either train_size or n_splits should be defined"

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_bad_window_type(X, y, group_numbers):
    cv_args = {
        "test_size": 1,
        "train_size": 3,
        "window_type": "incorrect_window_type",
    }
    expected_results = None
    error_message = 'Window type can be either "rolling" or "expanding"'

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_train_size_with_expanding_window(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 3, "window_type": "expanding"}
    expected_results = None
    error_message = "Train size can be specified only with rolling window"

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_not_specified_groups(X, y):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = None
    error_message = "The groups should be specified"

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, None, cv_args, expected_results)


def test_not_consecutive_group_numbers(X, y, not_consecutive_group_numbers):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = None
    error_message = "The groups should be consecutive"

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, not_consecutive_group_numbers, cv_args, expected_results)


def test_not_consecutive_group_names(X, y, not_consecutive_group_names):
    cv_args = {"test_size": 1, "train_size": 3}
    expected_results = None
    error_message = "The groups should be consecutive"

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, not_consecutive_group_names, cv_args, expected_results)


def test_too_large_train_size_(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 10}
    expected_results = None
    error_message = (
        r"Not enough data to split number of groups \(6\)"
        r" for number splits \(-4\) with train size \(10\),"
        r" test size \(1\), gap size \(0\), shift size \(1\)"
    )

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_too_large_n_splits(X, y, group_numbers):
    cv_args = {"test_size": 1, "n_splits": 10}
    expected_results = None
    error_message = (
        r"Not enough data to split number of groups \(6\)"
        r" for number splits \(10\) with train size \(-4\),"
        r" test size \(1\), gap size \(0\), shift size \(1\)"
    )

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_too_large_train_size_n_splits(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 10, "n_splits": 10}
    expected_results = None
    error_message = (
        r"Not enough data to split number of groups \(6\)"
        r" for number splits \(10\) with train size \(10\),"
        r" test size \(1\), gap size \(0\), shift size \(1\)"
    )

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_too_large_shift_size(X, y, group_numbers):
    cv_args = {"test_size": 1, "n_splits": 3, "shift_size": 10}
    expected_results = None
    error_message = (
        r"Not enough data to split number of groups \(6\)"
        r" for number splits \(3\) with train size \(-15\),"
        r" test size \(1\), gap size \(0\), shift size \(10\)"
    )

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_too_large_gap_size(X, y, group_numbers):
    cv_args = {"test_size": 1, "n_splits": 3, "gap_size": 10}
    expected_results = None
    error_message = (
        r"Not enough data to split number of groups \(6\)"
        r" for number splits \(3\) with train size \(-7\),"
        r" test size \(1\), gap size \(10\), shift size \(1\)"
    )

    with pytest.raises(ValueError, match=error_message):
        check_splits(X, y, group_numbers, cv_args, expected_results)


def test_cross_val_score(X, y, group_numbers):
    cv_args = {"test_size": 1, "train_size": 3}
    cv = GroupTimeSeriesSplit(**cv_args)

    expected_scores = np.array([0, 0.5, 0.25])
    clf = DummyClassifier(strategy="most_frequent")
    scoring = "accuracy"
    cv_scores = cross_val_score(clf, X, y, groups=group_numbers, scoring=scoring, cv=cv)

    assert np.array_equal(cv_scores, expected_scores)
